//
// Copyright (c) 2001-2010 by PDFTron Systems Inc. All Rights Reserved.
//

using System;
using System.IO;
using pdftron;
using pdftron.Common;
using pdftron.Filters;
using pdftron.SDF;
using pdftron.PDF;

// The following sample illustrates how to read/write a PDF document from/to 
// a memory buffer.  This is useful for applications that work with dynamic PDF
// documents that don't need to be saved/read from a disk.
namespace PDFDocMemoryTestCS
{
	class Class1
	{
		[STAThread]
		static void Main(string[] args)
		{
			PDFNet.Initialize();

			// Relative path to the folder containing test files.
			string input_path =  "../../../../TestFiles/";
			string output_path = "../../../../TestFiles/Output/";

			try  
			{
				// Read a PDF document in a memory buffer.
				FileStream fileStream1 = new FileStream(input_path + "tiger.pdf", FileMode.Open, FileAccess.Read);
				BinaryReader reader1 = new BinaryReader(fileStream1);
				byte[] file_buff = reader1.ReadBytes((int)reader1.BaseStream.Length);
				PDFDoc doc = new PDFDoc(file_buff, file_buff.Length);
				doc.InitSecurityHandler();

				int num_pages = doc.GetPageCount();

				ElementWriter writer = new ElementWriter();
				ElementReader reader = new ElementReader();
				Element element;

				// Perform some document editing ...
				// Here we simply copy all elements from one page to another.
				for(int i = 1; i <= num_pages; ++i)
				{
					Page pg = doc.GetPage(2 * i - 1);

					reader.Begin(pg);
					Page new_page = doc.PageCreate(pg.GetMediaBox());
					doc.PageInsert(doc.GetPageIterator(2*i), new_page);

					writer.Begin(new_page);
					while ((element = reader.Next()) != null) 	// Read page contents
					{
						writer.WriteElement(element);
					}

					writer.End();
					reader.End();
				}

				doc.Save(output_path + "doc_memory_edit.pdf", SDFDoc.SaveOptions.e_remove_unused);

				// Save the document to a memory buffer.
				byte[] buf = new byte[1]; 
				int buf_sz = 1;
				doc.Save(ref buf, ref buf_sz, SDFDoc.SaveOptions.e_remove_unused);

				// Write the contents of the buffer to the disk
				FileStream fileStream2 = new FileStream(output_path + "doc_memory_edit.txt", FileMode.Create, FileAccess.Write);
				BinaryWriter writer2 = new BinaryWriter(fileStream2);
				writer2.Write(buf);

				// Read some data from the file stored in memory
				reader.Begin(doc.GetPage(1));
				while ((element = reader.Next()) != null) {
					if (element.GetType() == Element.Type.e_path)
						Console.Write("Path, ");
				}
				reader.End();

				// Calling Dispose() on ElementReader/Writer/Builder can result in increased performance and lower memory consumption.
				reader.Dispose();
				writer.Dispose();

				Console.WriteLine("\n\nDone. Result saved in doc_memory_edit.pdf and doc_memory_edit.txt ...");
				doc.Close();
			}
			catch (PDFNetException e)
			{
				Console.WriteLine(e.Message);
			}
		}
	}
}
