using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using pdftron;
using pdftron.PDF;

namespace PDFViewCS
{
#if CUSTOM_NAV

	/// <summary>
	/// ThumbView is provided as an illustration of how to derive a new class 
	/// from pdftron.PDF.PDFView. You can fully customize this class to meet 
	/// your specific application requirements.
	/// 
	/// Note: 
	/// A mouse click on the page thumbnail changes the current page number 
	/// in the main PDF view. 
	/// </summary>
	public class ThumbView : PDFViewCtrl
	{
		public ThumbView() : base() {
			// For faster, but lower quality rendering uncomment the following line:
			SetImageSmoothing(false); 
			ShowNavPanel(false);
			SetToolMode(PDFViewCtrl.ToolMode.e_custom);
		}

		private pdftron.PDF.PDFViewCtrl _main_view = null;

		public void SetDoc(PDFDoc doc, PDFViewCtrl main_view) { 
			base.SetDoc(doc);
			_main_view = main_view; 
		}

		protected override void OnPaint(PaintEventArgs e)
		{			
			base.OnPaint(e); // Draw PDF thumbnails as a background.

			if (_main_view != null)
			{
				int cur_page = _main_view.GetCurrentPage();
				if (cur_page < 1) return;
				pdftron.PDF.Page page = GetDoc().GetPage(cur_page);

				// Draw a red border around the selected page.
				pdftron.PDF.Rect bbox = page.GetCropBox();
				bbox.Normalize();
				double left=bbox.x1, bottom=bbox.y1, right=bbox.x2, top=bbox.y2, tmp;
		
				ConvPagePtToScreenPt(ref left, ref bottom, cur_page);
				ConvPagePtToScreenPt(ref right, ref top, cur_page);
				if (top<bottom) { tmp=top; top=bottom; bottom=tmp; }
				if (right<left) { tmp=right; right=left; left=tmp; }
				e.Graphics.DrawRectangle(Pens.Red, (float)(left-2), (float)(bottom-2), (float)(right-left+4), (float)(top-bottom+4));
			}
		}

		protected override void OnMouseDown(MouseEventArgs e) 
		{
			if (_main_view != null)
			{
				int cur_page = GetPageNumberFromScreenPt(e.X, e.Y);
				if (cur_page < 1) return;
				else _main_view.SetCurrentPage(cur_page);
				Invalidate();
			}
		}

		protected override void OnPaintBackground(PaintEventArgs pevent) {}
	}

#endif 
}
